require 'amrita2/core'

module TableView
  include Amrita2::Core

  class ViewSetting
    include Enumerable
    def initialize(&block)
      @setting = {}
      def @setting.method_missing(sym, *args, &block) 
        if block_given?
          self[sym] = block
        else
          self[sym] = args[0]
        end
      end

      @setting.instance_eval &block

      class << @setting
        undef method_missing
      end
      create_class_for_item
    end

    def [](arg)
      @setting[arg]
    end

    def create_item(data)
      @item_class.new(self,data)
    end

    private
    def create_class_for_item
      src = @setting.collect do |k,v|
        case v
        when Proc
          <<-END
          def #{k}
              @view[:#{k}].call(@data)
          end
          END
        else
          <<-END
          def #{k}
              @data.#{v.to_s}
          end
          END
        end          
      end.join("\n")
      #puts src
      @item_class = Class.new(Item)
      @item_class.module_eval src
    end
  end

  class Item
    attr_reader :data
    def initialize(view, data)
      @view, @data = view, data
    end

    def inspect
      "Item for #{@data.inspect}"
    end

    def row_header
      nil
    end

    def col_header
      nil
    end
  end

  class PageData
    attr_reader :row_headers, :col_headers
    def initialize(items)
      @data = []
      @row_headers = []
      @row_headers_hash = {}
      @col_headers = []
      @col_headers_hash = {}
      items.each do |item|
        add_item(item)
      end
    end

    def add_item(item)
      row_index = get_row_index(item)
      col_index = get_col_index(item)
      add_item_at(row_index, col_index, item)
    end

    def get_row_index(item)
      i = @row_headers_hash[item.row]
      unless i
        i = @row_headers_hash[item.row] = @row_headers.size
        @row_headers << (item.row_header || item.row)
      end
      i
    end

    def get_col_index(item)
      i = @col_headers_hash[item.column]
      unless i
        i = @col_headers_hash[item.column] = @col_headers.size
        @col_headers << (item.col_header || item.column)
      end
      i
    end

    def add_item_at(r,c,item)
      @data[r] ||= []
      @data[r][c] ||= []
      @data[r][c] << item
    end

    def rows
      @data
    end

    def [](r,c=nil)
      rd = @data[r]
      if c
        if rd
          rd[c] or []
        else
          []
        end
      else
        []
      end
    end

    def format(m, &block)
      m.table do |mt| 
        mt.header do |mh|
          mh.first_col_header("")
          col_headers.each do |ch|
            mh.col_header(ch)
          end
        end
        row_headers.each_with_index do |row_header, row_index|
          mt.detail do |md|
            md.row_header(row_header)
            col_headers.size.times do |i|
              md.data(yield(self[row_index, i]))
            end
          end
        end
      end
    end

  end

    m_table = TagMatcher.new(:table)
    m_tr = TagMatcher.new(:tr)
    m_th = TagMatcher.new(:th)
    m_td = TagMatcher.new(:td)

    TABLE_SPEC = Amrita2::define_template_spec do 
      dynamic_element(:table, :matcher=>m_table) do
        dynamic_element(:header,  :matcher=>m_tr) do
          dynamic_element(:first_col_header,  :matcher=>m_th)
          dynamic_element(:col_header,  :matcher=>m_th)
        end
        dynamic_element(:detail,  :matcher=>m_tr) do
          dynamic_element(:row_header,  :matcher=>m_th)
          dynamic_element(:data,  :matcher=>m_td)
        end
      end
    end
end
